// Geometric Tools LLC, Redmond WA 98052
// Copyright (c) 1998-2014
// Distributed under the Boost Software License, Version 1.0.
// http://www.boost.org/LICENSE_1_0.txt
// http://www.geometrictools.com/License/Boost/LICENSE_1_0.txt
// File Version: 1.7.0 (2014/12/08)

//----------------------------------------------------------------------------
template <int N, typename Real>
Line<N, Real>::Line()
{
    origin.MakeZero();
    direction.MakeUnit(0);
}
//----------------------------------------------------------------------------
template <int N, typename Real>
Line<N, Real>::Line(Vector<N, Real> const& inOrigin,
    Vector<N, Real> const& inDirection)
    :
    origin(inOrigin),
    direction(inDirection)
{
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator==(Line const& line) const
{
    return origin == line.origin && direction == line.direction;
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator!=(Line const& line) const
{
    return !operator==(line);
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator<(Line const& line) const
{
    if (origin < line.origin)
    {
        return true;
    }

    if (origin > line.origin)
    {
        return false;
    }

    return direction < line.direction;
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator<=(Line const& line) const
{
    return operator<(line) || operator==(line);
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator>(Line const& line) const
{
    return !operator<=(line);
}
//----------------------------------------------------------------------------
template <int N, typename Real>
bool Line<N, Real>::operator>=(Line const& line) const
{
    return !operator<(line);
}
//----------------------------------------------------------------------------
